<?php
/**
 * Created by PhpStorm.
 * User: tuliomagalhaes
 * Date: 31/03/2015
 * Time: 14:34
 */

namespace Controller;

use Teknisa\Libs\Util\OAuthCustom;
use Util\InstanceProvider;
use Util\Messages;
use Zeedhi\DevOps\Util\Utilities;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\HTTP\Kernel;
use Zeedhi\Framework\Log\Exception;

class ApiProjectManagement extends Simple {

    private $oauth;
    /** @var \Zeedhi\Framework\HTTP\Kernel $kernel */
    private $kernel;
    /** @var \Service\ApiProjectManagement $apiService */
    private $apiService;
    /** @var \Controller\Project $projectController */
    private $projectController;
    /** @var \Controller\Task $taskController */
    private $taskController;
    /** @var \Controller\Appropriation $persistAppropriation */
    private $appropriationController;
    /** @var \Service\Task $taskService */
    private $taskService;
    /** @var \Service\Project $projectService */
    private $projectService;
    /**
     * @var Acl
     */
    private $aclController;
    /** @var WorkedTime */
    private $workedTimeController;

    public function __construct(OAuthCustom $oauth, Kernel $kernel, \Service\ApiProjectManagement $apiService,
                                Project $projectController, Task $taskController, Appropriation $appropriationController,
                                Acl $aclController, \Service\Task $taskService, \Service\Project $projectService,
                                WorkedTime $workedTimeController){
        $this->oauth = $oauth;
        $this->kernel = $kernel;
        $this->apiService = $apiService;
        $this->projectController = $projectController;
        $this->taskController = $taskController;
        $this->appropriationController = $appropriationController;
        $this->aclController = $aclController;
        $this->taskService = $taskService;
        $this->projectService = $projectService;
        $this->workedTimeController = $workedTimeController;
    }

    public function requestToken(Request $request, Response $response){
        try{
            $httpRequest = $this->kernel->getHttpRequest();
            $clientId = $httpRequest->get('CLIENT_ID');
            $clientSecret = $httpRequest->get('CLIENT_SECRET');
            $token = $this->oauth->grantAccessToken($clientId,$clientSecret,
                array('sessionId' => InstanceProvider::getSession()->getId()));
            $response->addDataSet(new DataSet('token', array($token)));
        }catch (\Exception $e){
            $response->addMessage(new Response\Message(\Util\Messages::ERR_SPONSOR_NOT_SAVED . $e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    public function addProject(Request\DataSet $request, Response $response){
        $projects = $request->getDataSet()->getRows();
        if(!empty($projects)){
            $persistedProjects = array();
            foreach ($projects as $project) {
                try{
                    $project = $this->apiService->buildProject($project);
                    $projectEntity = $this->projectController->persistProject($project);
                    $project["ID"] = $projectEntity->getId();
                    $persistedProjects[] = $project;
                } catch(\Exception $e) {
                    $msg = (empty($project['NAME']) ? "Project: {$project['NAME']}. \n" : "").$e->getMessage();
                    $response->addMessage(new Response\Message($msg, Response\Message::TYPE_ERROR));
                }
            }
            $response->addDataSet(new DataSet('projects', $persistedProjects));
        } else {
            $response->addMessage(new Response\Message('No project was provided.', Response\Message::TYPE_WARNING));
        }
    }

    public function addTask(Request\DataSet $request, Response $response){
        $tasks = $request->getDataSet()->getRows();
        if(!empty($tasks)){
            $persistedTasks = array();
            foreach ($tasks as $task) {
                try{
                    $task = $this->apiService->buildTask($task);
                    $taskEntity = $this->taskController->persistProjectTask($task);
                    $task['ID'] = $taskEntity->getId();
                    $persistedTasks[] = $task;
                } catch(\Exception $e) {
                    $msg = (empty($task['NAME']) ? "Task: {$task['NAME']}. \n" : "").$e->getMessage();
                    $response->addMessage(new Response\Message($msg, Response\Message::TYPE_ERROR));
                }
            }
            $response->addDataSet(new DataSet('tasks', $persistedTasks));
        } else {
            $response->addMessage(new Response\Message('No task was provided.', Response\Message::TYPE_WARNING));
        }
    }

    public function addAppropriation(Request\DataSet $request, Response $response){
        $appropriations = $request->getDataSet()->getRows();
        if(!empty($appropriations)){
            $persistedAppropriations = array();
            foreach ($appropriations as $appropriation) {
                try {
                    $appropriation = $this->apiService->buildAppropriation($appropriation);
                    $appropriationEntity = $this->appropriationController->persistAppropriation($appropriation);
                    $appropriation['ID'] = $appropriationEntity->getId();
                    $persistedAppropriations[] = $appropriation;
                } catch(\Exception $e) {
                    $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
                }
            }
            $response->addDataSet(new DataSet('appropriations', $persistedAppropriations));
        } else {
            $response->addMessage(new Response\Message('No appropriation was provided.', Response\Message::TYPE_WARNING));
        }
    }

    public function saveProfile(Request\DataSet $request, Response $response){
        $profiles = $request->getDataSet()->getRows();
        if(!empty($profiles)){
            $persistedProfiles = array();
            foreach ($profiles as $profile){
                try {
                    $profile = $this->apiService->buildProfile($profile);
                    $profileEntity = $this->aclController->persistProfile($profile);
                    $profile['ID'] = $profileEntity->getId();
                    $persistedProfiles[] = $profile;

                } catch(\Exception $e) {
                    $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
                }
            }
            $response->addDataSet(new DataSet('profiles', $persistedProfiles));
        }else{
            $response->addMessage(new Response\Message('No profiles was provided.', Response\Message::TYPE_WARNING));
        }
    }

    public function removeProfile(Request\DataSet $request, Response $response){
        $profiles = $request->getDataSet()->getRows();
        if(!empty($profiles)){
            $count = 0;
            foreach ($profiles as $profile){
                try {
                    if(!empty($profile['EXTERNAL_ID'])){
                        $profile['ID'] = $this->apiService->findProfileIdByExternalId($profile['EXTERNAL_ID']);
                    }
                    if(empty($profile['ID'])){
                        throw new \Exception('profiles not found');
                    }
                    $this->aclController->removeProfile($profile);

                } catch(\Exception $e) {
                    $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
                    $count--;
                }
                $count++;
            }
            $response->addMessage(new Response\Message("$count access profiles deleted"));
        }else{
            $response->addMessage(new Response\Message('No profiles was provided.', Response\Message::TYPE_WARNING));
        }
    }

    public function createProfileResource(Request\DataSet $request, Response $response){
        $profileResources = $request->getDataSet()->getRows();
        if(!empty($profileResources)){
            $count = 0;
            foreach ($profileResources as $profileResource){
                try {
                    $profileResource['PROFILE'] = $this->apiService->findProfileIdByExternalId($profileResource['PROFILE_EXTERNAL_ID']);
                    if(empty($profileResource['PROFILE'])){
                        throw new \Exception('Profile not found');
                    }
                    $profileResource['ID'] = $this->apiService->findProfileIdByExternalId($profileResource['PROFILE_EXTERNAL_ID']);
                    if(empty($profileResource['ID'])){
                        throw new \Exception('Resource not found');
                    }
                    $this->aclController->persistProfileResource($profileResource);

                } catch(\Exception $e) {
                    $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
                    $count--;
                }
                $count++;
            }
            $response->addMessage(new Response\Message("$count profile connections with resource created"));
        }else{
            $response->addMessage(new Response\Message('No profiles was provided.', Response\Message::TYPE_WARNING));
        }
    }

    public function removeProfileResource(Request\DataSet $request, Response $response){
        $profileResources = $request->getDataSet()->getRows();
        if(!empty($profileResources)){
            $count = 0;
            foreach ($profileResources as $profileResource){
                try {
                    $profileResource['PROFILE'] = $this->apiService->findProfileIdByExternalId($profileResource['PROFILE_EXTERNAL_ID']);
                    if(empty($profileResource['PROFILE'])){
                        throw new \Exception('Profile not found');
                    }
                    $profileResource['ID'] = $this->apiService->findProfileIdByExternalId($profileResource['PROFILE_EXTERNAL_ID']);
                    if(empty($profileResource['ID'])){
                        throw new \Exception('Resource not found');
                    }
                    $this->aclController->deleteProfileResource($profileResource);

                } catch(\Exception $e) {
                    $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
                    $count--;
                }
                $count++;
            }
            $response->addMessage(new Response\Message("$count profile connections with resource deleted"));
        }else{
            $response->addMessage(new Response\Message('No profiles was provided.', Response\Message::TYPE_WARNING));
        }
    }

    public function findTaskByUser(Request\Row $request, Response $response){
        try{
            $data = $request->getRow();
            if(!empty($data["RESOURCE"])){
                $beginDate  = empty($data["BEGIN_DATE"]) ? null : $data["BEGIN_DATE"];
                $endDate    = empty($data["END_DATE"]) ? null : $data["END_DATE"];
                $tasks      = $this->apiService->findTaskByUser($data["RESOURCE"], $beginDate, $endDate);
                $response->addDataSet(new DataSet('tasks', $tasks));
            } else {
                $response->addMessage(new Response\Message('Required field: RESOURCE.', Response\Message::TYPE_ERROR));
            }
        } catch (\Exception $e){
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    public function findResourceIdByEmail(Request\DataSet $request, Response $response){
        try{
            $data = $request->getDataSet()->getRows();
            $data = $data[0];

            if(!empty($data["RESOURCE"])){
                $id      = $this->apiService->findResourceIdByEmail($data["RESOURCE"]);
                $response->addDataSet(new DataSet('ResourceId', array($id)));

            } else {
                $response->addMessage(new Response\Message('Required field: RESOURCE.', Response\Message::TYPE_ERROR));
            }
        } catch (\Exception $e){
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    public function findAllManager(Request $request, Response $response){
        try{;


            $id      = $this->apiService->findAllManager();
            $response->addDataSet(new DataSet('ResourceId', array($id)));


        } catch (\Exception $e){
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    public function deleteResource(Request\DataSet $request, Response $response){
        $count = 0;
        $dataSet = $request->getDataSet()->getRows();
        foreach ($dataSet as $param) {
            try{
                $this->apiService->deleteResource($param['ID']);
            }catch(\Exception $ex){
                $response->addMessage(new Response\Message(\Util\Messages::ERR_NOT_SUCCESSFUL_DELETED_EXPLANATION . $ex->getMessage(), Response\Message::TYPE_ERROR));
                $count--;
            }
            $count++;
        }
        $response->addMessage(new Response\Message("$count recurso(s) deletado(s)."));
    }

    /* Customer Portal Integration */
    public function findProductCallClassification(Request $request, Response $response) {
        $issueClassification = $this->taskService->findIssueClassification();
        $response->addDataSet(new DataSet('findProductCallClassification', $issueClassification));
    }

    public function findProductCallProducts(Request\Filter $request, Response $response) {
            $products = $this->projectService->findProducts();
            $response->addDataSet(new DataSet('findProductCallProducts', $products));
    }

    public function createIssueByProductCall(Request\Row $request, Response $response) {
        $param = $request->getRow();
        $name           = $param["REFERENCE"];
        $classification = $param["CLASSIFICATION"];
        $description    = empty($param["DESCRIPTION"]) ? "" : $param["DESCRIPTION"];
        $board          = $param["BOARD"];
        $due_date       = $param['DUE_DATE'];
        $sponsor        = empty($param["SPONSOR"]) ? null : $param["SPONSOR"];
        $reporterCdoperador = empty($param["REPORTER_CDOPERADOR"]) ? null : $param["REPORTER_CDOPERADOR"];

        try{
            $issue = $this->apiService->createIssueByProductCall($name, $classification, $description, $board,
                $due_date,$sponsor,$reporterCdoperador);
            $response->addDataSet(new DataSet('issue', array($issue)));
        }
        catch(\Exception $e){
            $response->addMessage(new Response\Message(Messages::ERR_NOT_SUCCESSFUL_SAVED_EXPLANATION . $e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    public function updateIssueByProductCall(Request\Row $request, Response $response) {
        $param = $request->getRow();
        $id             = $param['ID'];
        $name           = $param["REFERENCE"];
        $classification = $param["CLASSIFICATION"];
        $description    = empty($param["DESCRIPTION"]) ? "" : $param["DESCRIPTION"];
        $board          = $param["BOARD"];
        $due_date       = $param['DUE_DATE'];

        try {
            $issue = $this->apiService->updateIssueByProductCall($id, $name, $classification, $description, $board, $due_date);
            $response->addDataSet(new DataSet('issue', array($issue)));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message(Messages::ERR_NOT_SUCCESSFUL_SAVED_EXPLANATION . $e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    public function removeIssueById(Request\Filter $request, Response $response) {
        $params = $request->getFilterCriteria()->getConditions();
        try {
            if(!empty($params[0])){
                $issueId = $params[0]["value"];
                $result = $this->apiService->removeIssueById($issueId);
                $response->addDataSet(new DataSet('removeIssueById', array($result)));
            } else {
                $response->addDataSet(new DataSet('removeIssueById', array(false)));
            }
        } catch(\Exception $e) {
            $response->addDataSet(new DataSet('removeIssueById', array(false)));
        }
    }

    public function getIssuesLastEvolutionDates(Request\Filter $request, Response $response) {
        $param = $request->getFilterCriteria()->getConditions();
        try{
            $issuesIds = $param[0]["value"];
            $dates = $this->apiService->getIssuesLastEvolutionDates($issuesIds);
            $response->addDataSet(new DataSet('issuesLastEvolutionDates', $dates));
        } catch(\Exception $e) {
            $response->addMessage(new Response\Message(Messages::ERR_NOT_POSSIBLE_TO_GET_EVOLUTION_DATE . $e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    public function linkDownloadBudgetSm(Request\Row $request, Response $response) {
        $row = $request->getRow();
        try {
            $issueId = $row['ISSUE_ID'];
            $linkDownloadBudgetSm = $this->apiService->linkDownloadBudgetSm($issueId);
            Utilities::setDataSetDefault($request,$response,$linkDownloadBudgetSm);
        } catch(\Exception $e) {
            $response->setError(new Response\Error(Messages::ERR_NOT_POSSIBLE_TO_GET_EVOLUTION_DATE . $e->getMessage(), $e->getCode()));
        }
    }

    public function saveCommentOnIssueByProductCallComment(Request\Row $request, Response $response) {
        $row = $request->getRow();
        try {
            $issueId = $row['ISSUE_ID'];
            $comment = $row['COMMENT'];
            $cdoperador = empty($row["CDOPERADOR"]) ? null : $row["CDOPERADOR"];
            $this->apiService->saveCommentOnIssuesByProductCallComment($issueId, $comment, $cdoperador);
        } catch(\Exception $e) {
            $response->addMessage(new Response\Message(Messages::ERR_NOT_POSSIBLE_TO_SAVE_COMMENT . $e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }


    public function saveReopenIssueByProductCall(Request\Row $request, Response $response) {
        $row = $request->getRow();
        try {
            $issueId = $row['ISSUE_ID'];
            $comment = $row['COMMENT'];
            $cdoperador = empty($row["CDOPERADOR"]) ? null : $row["CDOPERADOR"];
            $cdCliente = empty($row["CDCLIENTE"]) ? null : $row["CDCLIENTE"];
            $this->apiService->saveReopenIssueByProductCall($issueId, $comment, $cdoperador, $cdCliente);
        } catch(\Exception $e) {
            $response->addMessage(new Response\Message(Messages::ERR_NOT_POSSIBLE_TO_SAVE_COMMENT . $e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    public function saveRefuseIssueByProductCall(Request\Row $request, Response $response) {
        $row = $request->getRow();
        try {
            $issueId = $row['ISSUE_ID'];
            $comment = $row['COMMENT'];
            $cdoperador = empty($row["CDOPERADOR"]) ? null : $row["CDOPERADOR"];
            $document = empty($row["DOCUMENT"]) ? null : $row["DOCUMENT"];

            $this->apiService->saveRefuseIssueByProductCall($issueId, $comment, $cdoperador, $document);
        } catch(\Exception $e) {
            $response->addMessage(new Response\Message(Messages::ERR_NOT_POSSIBLE_TO_SAVE_COMMENT . $e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    public function saveApproveIssueByProductCall(Request\Row $request, Response $response) {
        $row = $request->getRow();
        try {
            $issueId = $row['ISSUE_ID'];
            $comment = $row['COMMENT'];
            $cdoperador = empty($row["CDOPERADOR"]) ? null : $row["CDOPERADOR"];
            $document = empty($row["DOCUMENT"]) ? null : $row["DOCUMENT"];
            $this->apiService->saveApproveIssueByProductCall($issueId, $comment, $cdoperador, $document);
        } catch(\Exception $e) {
            $response->addMessage(new Response\Message(Messages::ERR_NOT_POSSIBLE_TO_SAVE_COMMENT . $e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    public function saveGiveUpImplementationIssue(Request\Row $request, Response $response) {
        $row = $request->getRow();
        try {
            $issueId = $row['ISSUE_ID'];
            $comment = $row['COMMENT'];
            $cdoperador = empty($row["CDOPERADOR"]) ? null : $row["CDOPERADOR"];
            $document = empty($row["DOCUMENT"]) ? null : $row["DOCUMENT"];
            $this->apiService->saveGiveUpImplementationIssue($issueId, $comment, $cdoperador, $document);

        } catch(\Exception $e) {
            $response->setError(new Response\Error(Messages::ERR_NOT_POSSIBLE_TO_SAVE_COMMENT . $e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    public function findCommentsOnIssueForProductCall(Request\Filter $request, Response $response) {
        $params = $request->getFilterCriteria()->getConditions();
        if(empty($params[0])) {
            $response->addDataSet(new DataSet('findCommentsOnIssueForProductCall', array()));
        } else {
            $issueId  = $params[0]["value"];
            $comments = $this->apiService->findCommentsOnIssueForProductCall($issueId);
            $response->addDataSet(new DataSet('findCommentsOnIssueForProductCall', $comments));
        }
    }

    public function uploadIssueAttachments(Request\Row $request, Response  $response) {
        $row = $request->getRow();
        try {
            $issueId = $row['ISSUE_ID'];
            $documents = $row['DOCUMENTS'];
            $commentId = empty($row['COMMENT_ID']) ? null : $row['COMMENT_ID'];
            $cdoperador = empty($row["CDOPERADOR"]) ? null : $row["CDOPERADOR"];
            $attachmentsIds = $this->apiService->uploadIssueAttachments($issueId, $documents,$cdoperador, $commentId);
            $response->addDataSet(new DataSet("attachmentsIds", $attachmentsIds));
        } catch(\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    public function getIssueAttachmentsDataSet(Request\Filter $request, Response $response) {
        $params = $request->getFilterCriteria()->getConditions();
        if(!empty($params[0])) {
            $issueId = $params[0]['value'];
            $issueAttachments = $this->apiService->getIssueAttachmentsDataSet($issueId);
            $response->addDataSet(new DataSet('issueAttachments', $issueAttachments));
        } else {
            $response->addDataSet(new DataSet('issueAttachments', array()));
        }
    }
    public function saveWorkedTime(Request\DataSet $request, Response $response) {

        $dataSet = $request->getDataSet()->getRows();
        foreach ($dataSet as &$row) {
            $resourceId = $this->apiService->findResourceIdByEmail($row['EMAIL']);
            $row['changed'] = true;
            if($resourceId){
                $row['RESOURCE_T'] = $resourceId;
            }else{
                throw new Exception('E-mail não cadastrado ou usuario inativo ' . $row['EMAIL']);
            }
            if(!$row['WORK_DATE']){
                throw new Exception('O campo "WORK_DATE" não pode ser vazio');
            }
            $row['ID'] = null;
            if(!empty($row['IS_PENDING']) && $row['IS_PENDING'] !== 'N' ){
                $row['IS_PENDING'] = 'Y';
            }else{
                $row['IS_PENDING'] = 'N';
            }
        }
        $this->workedTimeController->save(new Request\DataSet(new DataSet($request->getDataSet()->getDataSourceName(), $dataSet),
            $request->getMethod(), $request->getRoutePath(), $request->getUserId()), $response,false);

    }

    public function apiFrequency(Request $request, Response $response) {

        /** @var Request\DataSet $request */
        $dataSet = $request->getDataSet()->getRows();
        foreach ($dataSet as $row) {
            $WorkedTime = $row['WORKED_TIME'];
            $this->saveWorkedTime(new Request\DataSet(new DataSet($request->getDataSet()->getDataSourceName(), $WorkedTime),
                $request->getMethod(), $request->getRoutePath(), $request->getUserId()), $response);
        }

    }

    public function getAttachmentsDataSet(Request\Filter $request, Response  $response) {

        try {
            $filters = $request->getFilterCriteria()->getConditions();
            $id = Utilities::getValueInFilterDataSourceByName($filters,'ID');
            $attachmentsIds = $this->apiService->getIssueById($id);
            $response->addDataSet(new DataSet("attachmentsIds", $attachmentsIds));
        } catch(\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param Request\Filter $request
     * @param Response $response
     */
    public function findTaskResource(Request\Filter $request, Response  $response)
    {
        try {
            $param = $request->getFilterCriteria()->getConditions();
            $filterstring = Utilities::getFilterTableColumn($param, 'NAME');
            $project = $param[0]['value'];
            $taskClassificationControl = Utilities::getValueInFilterDataSourceByName($param, 'TASK_CLASSIFICATION');
            $dataSet = InstanceProvider::getService()->getResource()->getResourceByProject($project, $filterstring, $taskClassificationControl);
            $response->addDataSet(new DataSet('taskResource', $dataSet));
        } catch(\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    public function findTaskParent(Request\Filter $request, Response  $response)
    {
        try {
            $param          = $request->getFilterCriteria()->getConditions();
            $filterString   = Utilities::getFilterTableColumn($param, 'NAME');
            $isParent       = DevOpsUtilities::getValueInFilterDataSourceByName($param,'IS_PARENT',true);
            $project        = DevOpsUtilities::getValueInFilterDataSourceByName($param,'PROJECT',true);
            $task           = DevOpsUtilities::getValueInFilterDataSourceByName($param,'TASK',true);
            $selectedTasks  = DevOpsUtilities::getValueInFilterDataSourceByName($param,'selectedTasksIds',true);

            if(!$project){
                $project = DevOpsUtilities::getValueInFilterDataSourceByName($param,'projectId',true);
            }else if(!$project){
                $project = '0';
            }

            if(!$task){
                $task = '0';
            }

            if(!$selectedTasks){
                $selectedTasks = array(0);
            }

            $dataSet = $this->taskService->getParentTaskByProjectAndTask($project, $task, $selectedTasks,$isParent,$filterString);
            $response->addDataSet(new DataSet('taskParent', $dataSet));
        } catch(\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    public function saveMultipleProjectTask(Request\DataSet $request, Response $response)
    {
        $data = $request->getDataSet()->getRows();
        try{
            $row = $data[0];
            $this->setSessionVariableFromDevOps($row);
            $tasks = $row['TASKS'];
            $isRelatedTask = empty($row['IS_RELATED_TASK']) ? true : $row['IS_RELATED_TASK'];
            $finishedTask = empty($row['FINISHED_TASK']) ? null : $row['FINISHED_TASK'];
            $document = empty($row['DOCUMENT']) ? null : $row['DOCUMENT'];
            $notSetTotalTime = empty($row['NOT_SET_TOTAL_TIME']) ? null : $row['NOT_SET_TOTAL_TIME'];
            $createdTaskIds = $this->taskService->saveMultipleProjectTask($tasks, $finishedTask, $isRelatedTask, $document,$notSetTotalTime);
            $response->addDataset(new DataSet("ID", $createdTaskIds));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * @param Request\DataSet $request
     * @param Response $response
     * @throws \Exception
     */
    public function commentsIssuesSubRelease(Request\DataSet $request, Response $response)
    {
        try {
            $releaseData = $request->getDataSet()->getRows();
            $errorLog = array();
            foreach ($releaseData as $row) {
                $this->setSessionVariableFromDevOps($row);
                $releaseId = $row['SUBRELEASE_ID'];
                $version = empty($row['VERSION'])? null :$row['VERSION'];
                $errorLog = InstanceProvider::getService()->getComment()->sendCommentsIssuesSubRelease($releaseId, $version);
            }
            $response->addDataset(new DataSet('errorLog', $errorLog));

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function sendCommentsApplyDeploy(Request\DataSet $request, Response $response) {
        try {
            $row           = $request->getDataSet()->getRows()[0];
            $server        = $row['SERVER'];
            $version       = $row['VERSION'];
            $currentUserId = empty($row['USER_ID']) ? null    : $row['USER_ID'];
            $issues        = empty($row['ISSUES'])  ? array() : $row['ISSUES'];
            $errorLog      = array();
            $fromDeploy    = true;

            if($currentUserId) {
                InstanceProvider::getEnvironment()->setCurrentUserId($currentUserId);
            }
            foreach ($issues as $issue){
                $errorLog = InstanceProvider::getService()->getComment()->sendsMailAdvisingAboutAvailableIssue($issue, '', $version,$errorLog, '', '', $server, $fromDeploy);
            }
            $response->addDataset(new DataSet('errorLog', $errorLog));
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function sendProductApplyDeploy(Request\DataSet $request, Response $response) {
        try {
            $row           = $request->getDataSet()->getRows()[0];
            $server        = $row['SERVER'];
            $products       = $row['PRODUCT'];
            $currentUserId = empty($row['USER_ID']) ? null    : $row['USER_ID'];
            $errorLog      = array();
            $fromDeploy    = true;

            if($currentUserId) {
                InstanceProvider::getEnvironment()->setCurrentUserId($currentUserId);
            }
            foreach ($products as $product){
                foreach($product['ISSUES'] as $issue){
                    $errorLog = InstanceProvider::getService()->getComment()->sendsMailAdvisingAboutAvailableIssue($issue, '', $product['VERSION'],$errorLog, '', '', $server, $fromDeploy);
            }

            }
            $response->addDataset(new DataSet('errorLog', $errorLog));
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function commentsIssuesRelease(Request\DataSet $request, Response $response)
    {
        try {
            $releaseData = $request->getDataSet()->getRows();
            $errorLog = array();
            foreach ($releaseData as $row) {
                $this->setSessionVariableFromDevOps($row);
                $releaseId = $row['RELEASE_ID'];
                $version = empty($row['VERSION'])? null :$row['VERSION'];
                $errorLog = InstanceProvider::getService()->getComment()->sendCommentsIssuesRelease($releaseId,$version);
            }
            $response->addDataset(new DataSet('errorLog', $errorLog));

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param Request\DataSet $request
     * @param Response $response
     */
    public function saveMultipleProjectTaskAndCreateRelationship(Request\DataSet $request, Response $response)
    {
        $data = $request->getDataSet()->getRows();
        try{
            $param = $data[0];
            $this->setSessionVariableFromDevOps($param);
            $tasks = $param['TASKS'];
            $isRelatedTask = empty($param['IS_RELATED_TASK']) ? true : $param['IS_RELATED_TASK'];
            $finishedTask = empty($param['FINISHED_TASK']) ? null : $param['FINISHED_TASK'];
            $document = empty($param['DOCUMENT']) ? null : $param['DOCUMENT'];
            $notSetTotalTime = empty($param['NOT_SET_TOTAL_TIME']) ? null : $param['NOT_SET_TOTAL_TIME'];
            $createdTaskIds = $this->taskService->saveMultipleProjectTask($tasks, $finishedTask, $isRelatedTask, $document,$notSetTotalTime);
            $response->addDataset(new DataSet("ID", $createdTaskIds));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function saveProjectTask(Request\DataSet $request, Response $response)
    {
        $data = $request->getDataSet()->getRows();
        try{
            $param = $data[0];
            $validationSave = InstanceProvider::getTaskController()->validateSaveProjectTask($param['PROJECT'], $param['PARENT']);
            if($validationSave) {
                $taskEntity = InstanceProvider::getTaskController()->persistProjectTask($param);
                $environment = InstanceProvider::getEnvironment();
                $taskPaths  = InstanceProvider::getService()->getAppropriation()->findPaths($taskEntity->getId(), $environment->getCurrentUserId());
                $response->addDataset(new DataSet("ID", array($taskEntity->getId(), $taskPaths)));
            } else {
                throw new \Exception(Messages::ERR_NOT_SAVE_TASK_INCOERENT_VALUES,1);
            }
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }

    }

    public function setSessionVariableFromDevOps($row)
    {
        $utitlities = InstanceProvider::getEnvironment();
        $utitlities->setCurrentOrganizationId($row['ORGANIZATION_ID']);
        $utitlities->setCurrentUserId($row['RESOURCE_ID']);
        $user = $utitlities->getCurrentUser();
        $externalCode = $user->getExternalCode();
        $utitlities->setCurrentCdoperador($externalCode);
    }
}
