<?php
namespace Zeedhi\DevOps\Model\Projectmanagement\Repositories;

use Doctrine\ORM\EntityRepository;

class Appropriation extends EntityRepository {

	public function findAppropriationByTask($idTask, $idResource, $idOrganization, $date) {

		$query = <<<SQL
			SELECT 	TRUNC(AP.TOTAL/3600)||':'|| SUBSTR(TO_CHAR(100 +MOD(TRUNC(AP.TOTAL/60),60)),2,2) AS TOTAL, AP.NOTE,
					TO_CHAR(AP.END_HOUR, 'DD/MM/YYYY HH24:MI') AS END_HOUR, TO_CHAR(AP.BEGIN_HOUR,'DD/MM/YYYY HH24:MI') AS BEGIN_HOUR, AP.DESCRIPTION,
					NVL(AP.COMPLETED, 0) || '%' AS PERCENT_COMPLETED, AP.ID, AL.ID AS ALLOCATION
			FROM APPROPRIATION AP
			INNER JOIN ALLOCATION AL ON AL.TASK = :idTask AND AL.RESOURCE_T = :idResource
			WHERE 	AP.TASK = :idTask
			AND 	AP.RESOURCE_T = :idResource
			AND 	AP.ORGANIZATION = :idOrganization
SQL;
		if ($date) {
			$query .= " AND TO_CHAR(CREATED_AT,'DD-MM-YYYY') = '$date'";
		}

		$param = array(
			'idTask' => $idTask,
			'idResource' => $idResource,
			'idOrganization' => $idOrganization
		);

		$query .=" ORDER BY ID";
		return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
	}

    public function validateApropriationByTask($idTask) {

		$query = <<<SQL
			SELECT 	TRUNC(AP.TOTAL/3600)||':'|| SUBSTR(TO_CHAR(100 +MOD(TRUNC(AP.TOTAL/60),60)),2,2) AS TOTAL, AP.NOTE,
					TO_CHAR(AP.END_HOUR, 'DD/MM/YYYY HH24:MI') AS END_HOUR, TO_CHAR(AP.BEGIN_HOUR,'DD/MM/YYYY HH24:MI') AS BEGIN_HOUR, AP.DESCRIPTION,
					NVL(AP.COMPLETED, 0) || '%' AS PERCENT_COMPLETED, AP.ID
			FROM    APPROPRIATION AP
			WHERE 	AP.TASK = :idTask
SQL;
		$param = array(
			'idTask' => $idTask
		);

		return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
	}

	public function getAppropriationsByTaskTimeControl($idTask, $idResource, $idOrganization, $filters) {
        $query = <<<SQL
        SELECT  TOTAL, BEGIN_HOUR, END_HOUR, PERCENT_COMPLETED, DESCRIPTION, USETIMEPLANNING, ID, NOTE, MAX(ALLOCATION) AS ALLOCATION , ORDER_FRONT, BEGIN_HOUR_ORDER
          FROM (
            SELECT TOTAL, BEGIN_HOUR, END_HOUR, PERCENT_COMPLETED, DESCRIPTION, USETIMEPLANNING, ID, NOTE, ALLOCATION, ORDER_FRONT, BEGIN_HOUR_ORDER
              FROM (
              SELECT PT.USETIMEPLANNING, AP.ID ,SUBSTR(TO_CHAR(100 +MOD(TRUNC(AP.TOTAL/3600),60)),2,2)||':'|| SUBSTR(TO_CHAR(100 +MOD(TRUNC(AP.TOTAL/60),60)),2,2) AS TOTAL, AP.NOTE,
                     CASE WHEN PT.USETIMEPLANNING = 'N' THEN TO_CHAR(AP.END_HOUR, 'D DD/MM/YYYY')ELSE TO_CHAR(AP.END_HOUR, 'D DD/MM/YYYY HH24:MI')END AS END_HOUR,
                     CASE WHEN PT.USETIMEPLANNING = 'N' THEN TO_CHAR(AP.BEGIN_HOUR, 'D DD/MM/YYYY')ELSE TO_CHAR(AP.BEGIN_HOUR, 'D DD/MM/YYYY HH24:MI')END AS BEGIN_HOUR,
                     AP.DESCRIPTION, NVL(AP.COMPLETED, 0) || '%' AS PERCENT_COMPLETED, AL.ID AS ALLOCATION,
                     TO_CHAR(AP.BEGIN_HOUR, 'YYYYMMDDHH24MISS') AS ORDER_FRONT, AP.BEGIN_HOUR AS BEGIN_HOUR_ORDER
                FROM APPROPRIATION AP
                JOIN TASK TA ON (TA.ID = AP.TASK)
                JOIN PROJECT PR ON (PR.ID = TA.PROJECT)
           LEFT JOIN PROJECT_TYPE PT ON (PT.ID = PR.PROJECT_TYPE)
           LEFT JOIN ALLOCATION AL ON (AL.TASK = :idTask AND AL.RESOURCE_T = :idResource)
               WHERE AP.TASK = :idTask
                 AND AP.RESOURCE_T = :idResource
                 AND AP.ORGANIZATION = :idOrganization
                 $filters))
      GROUP BY TOTAL, BEGIN_HOUR, END_HOUR, PERCENT_COMPLETED, DESCRIPTION, USETIMEPLANNING, ID, NOTE, ORDER_FRONT, BEGIN_HOUR_ORDER
      ORDER BY BEGIN_HOUR_ORDER DESC, ID DESC
SQL;

        $param = array(
			'idTask' => $idTask,
			'idResource' => $idResource,
			'idOrganization' => $idOrganization
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
	}

	public function getAppropriationData($appropriationId){
		$query = <<<SQL
			SELECT AP.ID,SUBSTR(TO_CHAR(100 +MOD(TRUNC(AP.TOTAL/3600),60)),2,2)||':'|| SUBSTR(TO_CHAR(100 +MOD(TRUNC(AP.TOTAL/60),60)),2,2) AS TOTAL, AP.NOTE,
            CASE WHEN PT.USETIMEPLANNING = 'N' THEN TO_CHAR(AP.END_HOUR, 'DD/MM/YYYY')ELSE TO_CHAR(AP.END_HOUR, 'DD/MM/YYYY HH24:MI')END AS END_HOUR,
            CASE WHEN PT.USETIMEPLANNING = 'N' THEN TO_CHAR(AP.BEGIN_HOUR, 'DD/MM/YYYY')ELSE TO_CHAR(AP.BEGIN_HOUR, 'DD/MM/YYYY HH24:MI')END AS BEGIN_HOUR,
                            AP.DESCRIPTION,NVL(AP.COMPLETED, 0) AS PERCENT_COMPLETED, AP.ID, AL.ID AS ALLOCATION, TA.NAME AS TASK_NAME, PR.ID AS PROJECT_ID,
                      PR.NAME AS PROJECT_NAME,PT.ALLOWENDTASK,  TO_CHAR(TA.END_DATE, 'YYYYMMDD')AS INT_TASK_END_DATE,
                       TO_CHAR(TA.END_DATE, 'DD/MM/YYYY')AS TASK_END_DATE
            FROM      APPROPRIATION AP
            JOIN      TASK TA ON (TA.ID = AP.TASK)
            JOIN      PROJECT PR ON (PR.ID = TA.PROJECT)
            LEFT JOIN PROJECT_TYPE PT ON (PT.ID = PR.PROJECT_TYPE)
            LEFT JOIN ALLOCATION AL ON AL.TASK = AP.TASK AND AL.RESOURCE_T = AP.RESOURCE_T
            WHERE     AP.ID = :appropriationId
SQL;
		$params = array(
			'appropriationId' => $appropriationId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

    public function getAllAppropriations($organizationId,$filtersString = '',$onDemand = true,$filtersResourcesString = ''){
        if($onDemand){
            $query = <<<SQL
              SELECT ID, NAME, "_hasChild", CASE WHEN TOTAL_APPROPRIATION_DAILY = ',' THEN ''
   ELSE TOTAL_APPROPRIATION_DAILY END AS TOTAL_APPROPRIATION_DAILY
    FROM(SELECT ID, NAME, 'true' as "_hasChild",LISTAGG(DAY||','||TOTAL_APPROPRIATION, ';') WITHIN GROUP (ORDER BY NAME) AS TOTAL_APPROPRIATION_DAILY
                FROM (SELECT RES.ID , APR.DAY, RES.ID AS RESOURCE_T, SUM(APR.TOTAL_APPROPRIATION) AS TOTAL_APPROPRIATION, ORDER_FRONT,RES.NAME
                      FROM RESOURCE_T RES
                      LEFT JOIN (SELECT * FROM(SELECT RES.NAME, APR.RESOURCE_T,APR.TOTAL AS TOTAL_APPROPRIATION ,TO_CHAR(APR.BEGIN_HOUR, 'DD/MM/YYYY') AS DAY,
                              TO_CHAR(APR.BEGIN_HOUR, 'YYYYMMDD') AS ORDER_FRONT,RES.STATUS,APR.BEGIN_HOUR AS PERIOD,
                             PER.STRUCTURE,RES.ID
                        FROM RESOURCE_T RES
                        LEFT JOIN APPROPRIATION APR ON (RES.ID = APR.RESOURCE_T )
                        JOIN PERMISSION PER ON (PER.RESOURCE_T = RES.ID AND NVL(PER.TYPE,'R') = 'R' )
                        LEFT JOIN TASK TAS ON(TAS.ID = APR.TASK)
                       WHERE RES.CLASS = 'E'
                         AND RES.ORGANIZATION = :organizationId
                         AND RES.STATUS <> 'D'
                         AND APR.BEGIN_HOUR IS NOT NULL
                         AND TAS.STATUS <> 'D'
                         AND RES.STATUS = 'A')
                         WHERE 1=1
                        $filtersString )APR ON(RES.ID = APR.RESOURCE_T )
                         JOIN PERMISSION PER ON (PER.RESOURCE_T = RES.ID AND NVL(PER.TYPE,'R') = 'R' )
                         JOIN STRUCTURE STR ON (STR.ID = PER.STRUCTURE AND STR.STATUS = 'A')
                    WHERE RES.CLASS = 'E'
                      AND RES.ORGANIZATION = :organizationId
                       $filtersResourcesString
                 GROUP BY RES.ID , APR.DAY,  APR.ORDER_FRONT,RES.NAME
                 ORDER BY ORDER_FRONT DESC
                        )
            GROUP BY ID, NAME, 'true'
            ORDER BY NAME ASC)

SQL;
        }else {
            $query = <<<SQL
              SELECT ID AS PARENT, DAY, RESOURCE_T, SUM(TOTAL_APPROPRIATION) AS TOTAL_APPROPRIATION, ORDER_FRONT,DAY_FRONT
                FROM (SELECT * FROM(SELECT RES.NAME, APR.RESOURCE_T,APR.TOTAL AS TOTAL_APPROPRIATION ,TO_CHAR(APR.BEGIN_HOUR, 'DD/MM/YYYY') AS DAY,
                              TO_CHAR(APR.BEGIN_HOUR, 'YYYYMMDD') AS ORDER_FRONT,RES.STATUS,APR.BEGIN_HOUR AS PERIOD,
                             PER.STRUCTURE,TO_CHAR(APR.BEGIN_HOUR, 'D') AS DAY_FRONT,RES.ID
                        FROM APPROPRIATION APR
                        JOIN RESOURCE_T RES ON (RES.ID = APR.RESOURCE_T )
                        LEFT JOIN PERMISSION PER ON (PER.RESOURCE_T = RES.ID AND NVL(PER.TYPE,'R') = 'R' )
                        LEFT JOIN TASK TAS ON(TAS.ID = APR.TASK)
                       WHERE APR.ORGANIZATION = :organizationId
                         AND APR.BEGIN_HOUR IS NOT NULL
                         AND TAS.STATUS <> 'D'
                         AND RES.STATUS = 'A')
                         WHERE 1=1
                         $filtersString)
            GROUP BY ID, DAY, RESOURCE_T,  ORDER_FRONT,DAY_FRONT
            ORDER BY ORDER_FRONT DESC

SQL;
        }
		$params = array(
			'organizationId' => $organizationId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}
    public function getAllAppropriationsByDate($organizationId,$filtersString = ''){

            $query = <<<SQL
             SELECT TASK_NAME,RESOURCE_T ,TOTAL_APPROPRIATION ,DAY ,ORDER_FRONT ,PERIOD
 FROM(SELECT * FROM(SELECT TAS.NAME AS TASK_NAME, APR.RESOURCE_T,APR.TOTAL AS TOTAL_APPROPRIATION ,TO_CHAR(APR.BEGIN_HOUR, 'DD/MM/YYYY') AS DAY,
                              TO_CHAR(APR.BEGIN_HOUR, 'YYYYMMDD') AS ORDER_FRONT,APR.BEGIN_HOUR AS PERIOD
                        FROM APPROPRIATION APR
                        JOIN TASK TAS ON (TAS.ID = APR.TASK)
                       WHERE APR.ORGANIZATION = :organizationId
                         AND APR.BEGIN_HOUR IS NOT NULL
                         AND TAS.STATUS <> 'D')
                         WHERE 1=1
                        $filtersString )
            GROUP BY TASK_NAME, RESOURCE_T ,TOTAL_APPROPRIATION ,DAY ,ORDER_FRONT ,PERIOD
            ORDER BY ORDER_FRONT DESC

SQL;

		$params = array(
			'organizationId' => $organizationId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}
    public function getMaxPercentCompleted($taskId,$resourceId){
		$query = <<<SQL
        SELECT CASE WHEN MAX(AP.COMPLETED) IS NULL THEN 0 ELSE MAX(AP.COMPLETED) END AS PERCENT_COMPLETED
			FROM APPROPRIATION AP
			LEFT JOIN ALLOCATION AL ON AL.TASK = :taskId AND AL.RESOURCE_T = :resourceId
			WHERE 	AP.TASK = :taskId
			AND 	AP.RESOURCE_T = :resourceId
SQL;
		$params = array(
			'taskId' => $taskId,
			'resourceId' => $resourceId,
		);
		return intval($this->getEntityManager()->getConnection()->fetchAll($query, $params)[0]['PERCENT_COMPLETED']);
	}
	
	public function findDateLastAppropriation($userId){
		$query = <<<SQL
			SELECT MAX(BEGIN_HOUR) AS BEGIN_HOUR FROM APPROPRIATION WHERE RESOURCE_T = :USERID
SQL;
		$params = array(
			'USERID' => $userId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }
    
    public function getLatestAppropriationDate($taskId, $appropriationId){
        $whereAppId = "";
        if($appropriationId){
            $whereAppId = "AND ID <> :appropriationId";
        }
        $query = <<<SQL
        SELECT MAX(END_HOUR) AS END_HOUR FROM APPROPRIATION WHERE TASK = :taskID $whereAppId
SQL;
        $params = array(
            'taskID' => $taskId,
            'appropriationId' => $appropriationId
        );
    return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findAppropriationByTaskAndDate($userId, $taskId, $currentDate){
        $query = <<<SQL
            SELECT * 
              FROM APPROPRIATION
             WHERE TASK = :TASK_ID
               AND TO_CHAR(BEGIN_HOUR, 'DD/MM/YYYY') = :CURRENT_DATE
               AND RESOURCE_T = :USER_ID
SQL;
        $params = array(
            'USER_ID' => $userId,
            'TASK_ID' => $taskId,
            'CURRENT_DATE' => $currentDate
        );
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return isset($result[0]) ? $result[0] : $result;
    }

}